# Written by Bram Cohen
# see LICENSE.txt for license information

from zurllib import urlopen, quote
from httplib import InvalidURL
from btformats import check_peers
from bencode import bdecode
from threading import Thread, Lock
from time import time
from random import randrange, randint
from binascii import b2a_hex
from types import StringType
from urlparse import urlparse
from binascii import b2a_hex
import socket
import traceback

def toint(s):
    return long(b2a_hex(s), 16)

def tobinary(i):
    return (chr(i >> 24) + chr((i >> 16) & 0xFF) + 
        chr((i >> 8) & 0xFF) + chr(i & 0xFF))

class UDPError(Exception):
    def __init__(self, value=""):
        self.value = value
    def __str__(self):
        return repr(self.value)
        
class UDPRequester(object):
    def __init__(self, infohash, myid, port, key, ip):
        self.infohash = infohash
        self.myid = myid
        self.port = port
        self.key = key
        self.ip = ip
        
    def request(self, address):
        tx_id = randint(1,2**32)
        address = ("tracker.torrentbox.com", 2710)      
        msg = "\x00\x00\x00\x09\xB7\x21\x0C\x1C"  + tobinary(0) + tobinary(tx_id)
        
        try:
            osock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            osock.sendto(msg, address)
            print "addrs ", address, msg, len(msg)

            response = osock.recv(16)
            
            action = toint(response[:4])
            tx_id2 = toint(response[4:8])
            self.cid = response[8:16]
            if action != 0 or tx_id != tx_id2:
                raise UDPError("ERROR: %s" % response[8:])
                return -1
            
            print "response", {"action": toint(response[:4]), "txid": toint(response[4:8]), "cid": toint(response[8:16])}
        except:
            raise UDPError("UDP Socket Error - on request")
            return -1
        
        return self.announce(osock, address)
        
    def announce(self, osock, address):
        tx_id = randint(1,2**32)
        b = tobinary
        msg = self.cid + b(1) + b(tx_id) + self.infohash + self.myid + b(0) + b(0) + b(0) + b(0) + b(0) + b(0) + b(2)
        msg += b(0) + self.key + b(100) + b(self.port << 4)
      
        try:
            osock.sendto(msg, address)
            response = osock.recv(20*100)
            
            if len(response) < 20:
                raise UDPError("Invaid response from server - on announce")
                return -1
            
            action = toint(response[:4])
            tx_id2 = toint(response[4:8])
            interval = toint(response[8:12])
            leechers = toint(response[12:16])
            seeders = toint(response[16:20])
            
            if action != 1:
                raise UDPError("ERROR: %s" % response[8:])
                return -1
            
            for i in range(20, len(response), 6):
                peer = toint(response[i:i+4])
                port = toint("\x00\x00" + response[i+4:i+6])
                print (peer, port)
            
        except:
            traceback.print_exc()
            raise UDPError("UDP Socket Error - on announce")
            return -1
        
        print "done."
        return 1
        
        
class Rerequester(object):
    def __init__(self, url, interval, url_list, sched, howmany, minpeers, 
            connect, externalsched, amount_left, up, down,
            port, ip, myid, infohash, timeout, errorfunc, maxpeers, doneflag,
            upratefunc, downratefunc, ever_got_incoming):
        
        self.url_params = ('?info_hash=%s&peer_id=%s&port=%s&key=%s' %
            (quote(infohash), quote(myid), str(port),
            b2a_hex(myid[::-1][:4])))   
        if ip != '':
            self.url_params += '&ip=' + quote(ip)
        
        self.udper = UDPRequester(infohash, myid, port, myid[::-1][:4],ip)
        
        self.url_list = []
        if url_list == None:
            self.url_list.append(url)
        else:
            for ugroup in url_list:
                for urlstr in ugroup:
                    for url in urlstr.split(', '):
                        self.url_list.append(url)
                        
        self.interval = interval
        self.last = None
        self.trackerid = None
        self.announce_interval = 30 * 60
        self.sched = sched
        self.howmany = howmany
        self.minpeers = minpeers
        self.connect = connect
        self.externalsched = externalsched
        self.amount_left = amount_left
        self.up = up
        self.down = down
        self.timeout = timeout
        self.errorfunc = errorfunc
        self.maxpeers = maxpeers
        self.doneflag = doneflag
        self.upratefunc = upratefunc
        self.downratefunc = downratefunc
        self.ever_got_incoming = ever_got_incoming
        self.last_failed = True
        self.last_time = 0
        self.curr_url = ""

    def c(self):
        self.sched(self.c, self.interval)
        if self.ever_got_incoming():
            getmore = self.howmany() <= self.minpeers / 3
        else:
            getmore = self.howmany() < self.minpeers
        if getmore or time() - self.last_time > self.announce_interval:
            self.announce()

    def begin(self):
        self.sched(self.c, self.interval)
        self.announce(0)
    
    def get_url(self):
        url = self.url_list.pop(0)
        self.url_list.append(url)
        return url
   
    def announce(self, event = None, url=None):
        self.last_time = time()
        if url == None:
            url = self.get_url()

        self.curr_url = url
        urlstr = "%s%s" % (url, self.url_params)
        
        print 'announcing to', self.curr_url
        
        s = ('%s&uploaded=%s&downloaded=%s&left=%s' %
            (urlstr, str(self.up()), str(self.down()), str(self.amount_left())))
        if self.last is not None:
            s += '&last=' + quote(str(self.last))
        if self.trackerid is not None:
            s += '&trackerid=' + quote(str(self.trackerid))
        if self.howmany() >= self.maxpeers:
            s += '&numwant=0'
        else:
            s += '&compact=1'
        if event != None:
            s += '&event=' + ['started', 'completed', 'stopped'][event]
        
        set = SetOnce().set
        def checkfail(self = self, set = set):
            if set():
                if self.last_failed:
                    if self.upratefunc() < 100 and self.downratefunc() < 100:
                        self.errorfunc('Problem connecting to tracker [%s] - timeout exceeded' % self.curr_url, 'tracker_timeout')
                    else:
                        self.errorfunc('Problem connecting to tracker [%s] - timeout exceeded' % self.curr_url, 'tracker_bcool')
                self.last_failed = True
        self.sched(checkfail, self.timeout)
        Thread(target = self.rerequest, args = [s, set]).start()

    def rerequest(self, url, set):
        if url[:3] != "udp":
            return self.rerequest_http(url, set)
        else:
            return self.rerequest_http(url, set)
        
    def rerequest_udp(self, url, set):
        url = "http" + url[3:]  #for fucks sake!
        purl = urlparse(url)[1]
        purl = purl.split(":")
        #AF_INET address must be tuple, not list
        try:
            self.udper.request((purl[0], int(purl[1]))) 
        except (UDPError), e:
            if set():
                def fail_bcool(self = self, r = ('Problem connecting to tracker [%s] - %s' % (self.curr_url, str(e)))):
                    if self.last_failed:
                        self.errorfunc(r, 'tracker_bcool')
                    self.last_failed = True
                self.externalsched(fail_bcool, 0)

    def rerequest_http(self, url, set):
        try:
            h = urlopen(url)
            r = h.read()
            h.close()
            if set():
                def add(self = self, r = r):
                    self.last_failed = False
                    self.postrequest(r)
                self.externalsched(add, 0)
        except (IOError, socket.error, InvalidURL), e:
            if set():
                def fail_404(self = self, r = ('Problem connecting to tracker [%s] - %s' % (self.curr_url, str(e)))):
                    if self.last_failed:
                        self.errorfunc(r, 'tracker_404')
                    self.last_failed = True
                def fail_bcool(self = self, r = ('Problem connecting to tracker [%s] - %s' % (self.curr_url, str(e)))):
                    if self.last_failed:
                        self.errorfunc(r, 'tracker_bcool')
                    self.last_failed = True
                if self.upratefunc() < 100 and self.downratefunc() < 100:
                    self.externalsched(fail_404, 0)
                else:
                    self.externalsched(fail_bcool, 0)

    def postrequest(self, data):
        try:
            r = bdecode(data)
            check_peers(r)
            if r.has_key('failure reason'):
                if self.upratefunc() < 100 and self.downratefunc() < 100:
                    self.errorfunc(('rejected by tracker [%s] - %s' % (self.curr_url, r['failure reason'])), 'tracker_refuse')
                else:
                    self.errorfunc(('rejected by tracker [%s] - %s' % (self.curr_url, r['failure reason'])), 'tracker_bcool')
            else:
                if r.has_key('warning message'):
                    self.errorfunc(('warning from tracker [%s] - %s' % (self.curr_url, r['warning message'])), 'tracker_bcool')
                self.announce_interval = r.get('interval', self.announce_interval)
                self.interval = r.get('min interval', self.interval)
                self.trackerid = r.get('tracker id', self.trackerid)
                self.last = r.get('last')
                p = r['peers']
                peers = []
                if type(p) == type(''):
                    for x in xrange(0, len(p), 6):
                        ip = '.'.join([str(ord(i)) for i in p[x:x+4]])
                        port = (ord(p[x+4]) << 8) | ord(p[x+5])
                        peers.append((ip, port, None))
                else:
                    for x in p:
                        peers.append((x['ip'], x['port'], x.get('peer id')))
                ps = len(peers) + self.howmany()
                if ps < self.maxpeers:
                    if self.doneflag.isSet():
                        if r.get('num peers', 1000) - r.get('done peers', 0) > ps * 1.2:
                            self.last = None
                    else:
                        if r.get('num peers', 1000) > ps * 1.2:
                            self.last = None
                for x in peers:
                    self.connect((x[0], x[1]), x[2])
        except ValueError, e:
            if data != '':
                if self.upratefunc() < 100 and self.downratefunc() < 100:
                    self.errorfunc(('bad data from tracker [%s] - %s' % (self.curr_url, str(e))), 'tracker_refuse')
                else:
                    self.errorfunc(('bad data from tracker [%s] - %s' % (self.curr_url, str(e))), 'tracker_bcool')

        
class SetOnce(object):
    def __init__(self):
        self.lock = Lock()
        self.first = True

    def set(self):
        try:
            self.lock.acquire()
            r = self.first
            self.first = False
            return r
        finally:
            self.lock.release()

