#-----------------------------------------------------------------------------
# Name:        g3listctrl.py
# Purpose:     
#
# Author:      Jeremy Arendt
#
# Created:     2004/23/02
# RCS-ID:      $Id: g3listctrl.py $
# Copyright:   (c) 2002
# Licence:     See G3.LICENCE.TXT
#-----------------------------------------------------------------------------
import wx
import wx.html as html
from wx.lib.mixins.listctrl import ColumnSorterMixin, ListCtrlAutoWidthMixin
from btconfig import BTConfig
from images import Images
from g3listctrl import *
from feedparser import parse
from g3unihack import *

class RSSList(G3ListCtrl, ListCtrlAutoWidthMixin, ColumnSorterMixin):
    def __init__(self, parent, btconfig, bmps=None, onclickfunc=None, ondblclickfunc=None):
        G3ListCtrl.__init__(self, parent, btconfig, "RSSList", size=(100,100))
        self.CmdOnClick = onclickfunc
        self.CmdDblClick = ondblclickfunc
        
        ListCtrlAutoWidthMixin.__init__(self)
        ColumnSorterMixin.__init__(self, 2)
        self.itemDataMap = {}
        self.list_rows = {}


        if bmps == None:
            bmps = Images(self.btconfig['path'])
        
        self.pl_unchoked = wx.Color(255,255,240)
            
        self.images = [1] * 4
        self.i_list = wx.ImageList(12, 12)
        mask_color = wx.Color(255,255,255)
        b_mask_color = wx.Color(0,0,0)
        self.images[0] = self.i_list.AddWithColourMask(bmps.GetImage('blank.bmp'), b_mask_color)
        self.images[1] = self.i_list.AddWithColourMask(bmps.GetImage('usergreen.png'), mask_color)
        self.images[2] = self.i_list.AddWithColourMask(bmps.GetImage('userblue.png'), mask_color)
        self.images[3] = self.i_list.AddWithColourMask(bmps.GetImage('userred.png'), mask_color)

        self.SetImageList(self.i_list, wx.IMAGE_LIST_SMALL)
        
        self.col2sort = 1
        self.ascending = False
        
        cols = [ [True, "Title", wx.LIST_FORMAT_LEFT, 300],
                 [True, "URL", wx.LIST_FORMAT_LEFT, 400],
            ]

        self.InsertColumns(cols)
        self.SetEbedCol(1)
      
##        wx.EVT_LIST_COL_CLICK(self, -1, self.OnColClick)
##        wx.EVT_COMMAND_RIGHT_CLICK(self, -1, self.OnRightClick) # wxMSW
##        wx.EVT_RIGHT_UP(self, self.OnRightClick) # wxGTK  

        wx.EVT_LIST_ITEM_ACTIVATED(self, -1, self.OnDblClick)  
        wx.EVT_COMMAND_LEFT_CLICK(self, -1, self.OnClick) # wxMSW
        wx.EVT_LEFT_UP(self, self.OnClick) # wxGTK  

    def GetSelectedData(self):
        if self.GetFirstSelected() != -1:
            return self.GetItemData( self.GetFirstSelected() )
        else:
            return None

    def OnDblClick(self, event):
        if self.CmdDblClick:
            key = self.GetSelectedData()
            if self.itemDataMap.has_key(key):
                self.CmdDblClick(key)
        
    def OnClick(self, event):
        if self.CmdDblClick:
            key = self.GetSelectedData()
            if self.itemDataMap.has_key(key):
                self.CmdOnClick(key)
    
    def GetListCtrl(self):
        return self


    def Populate(self, rowdata):
        self.DeleteAllItems()
        for row in rowdata:
            key = hash(row)
            self.itemDataMap[key] = row
            item_idx = self.InsertRow(key, row)

class HTMLDescrption(html.HtmlWindow):
    def __init__(self, parent, btconfig, addfunc=None):
        html.HtmlWindow.__init__(self, parent, -1, size=(-1,80), style=wx.SUNKEN_BORDER|wx.NO_FULL_REPAINT_ON_RESIZE)
        self.SetBorders(4)
        self.CmdAddFunc = addfunc
    
    def OnLinkClicked(self, link):
        if ".torrent" in link.GetHref():
            if self.CmdAddFunc:
                self.CmdAddFunc(link.GetHref())
            
class RSSPanel(wx.Panel):
    def __init__(self, parent, btconfig, pos=wx.DefaultPosition, size=(100,100), 
            bmps = None, addfunc=None):
        wx.Panel.__init__(self, parent, -1)

        if addfunc:
            self.AddTorrent = addfunc
        
        self.rowdata = []
        self.descriptions = {}
        
        self.list = RSSList(self, btconfig, onclickfunc=self.OnClick, ondblclickfunc=self.OnDblClick)
        self.html = HTMLDescrption(self, btconfig, addfunc=self.AddTorrent)

        c = ["http://www.legaltorrents.com/rss.xml", "http://www.theinquirer.net/inquirer.rss"]
        self.editurl = wx.ComboBox(self, -1, "", size=(200,20), choices=c)
        self.editurl.SetValue(c[0])
        updatebutt = wx.Button(self, -1, "Update")

        topsizer = wx.FlexGridSizer()
        topsizer.AddGrowableCol(0)
        topsizer.Add(self.editurl, 1, wx.EXPAND)
        topsizer.Add(updatebutt)

        colsizer = wx.FlexGridSizer(cols=1)
        colsizer.AddGrowableRow(1)
        colsizer.AddGrowableCol(0)
        
        colsizer.Add(topsizer, 1, wx.EXPAND|wx.FIXED_MINSIZE)
        colsizer.Add(self.list, 1, wx.EXPAND|wx.FIXED_MINSIZE)
        colsizer.Add(self.html, 1, wx.EXPAND|wx.FIXED_MINSIZE)
        
        self.SetSizer(colsizer)
        self.Layout()

        wx.EVT_BUTTON(self, updatebutt.GetId(), self.Populate)
        wx.EVT_SIZE(self, self.OnSize)

    def AddTorrent(self, url):
        print url
    
    def OnSize(self, event):
        self.Layout()
        
    def OnClick(self, key):
        desc = self.descriptions.get(key)
        if desc:
            self.html.SetPage(desc[0])
    
    def OnDblClick(self, key):
        desc = self.descriptions.get(key)
        if desc:
            self.AddTorrent(desc[1])
                        
    def Populate(self, e=None):
        self.html.SetPage("")
        
        url = self.editurl.GetValue()
        i = 0
        found = False
        while i < self.editurl.GetCount():
            if self.editurl.GetString(i) == url:
                found = True
                break
            i += 1
        
        if not found:
            self.editurl.Insert(url, 0)

        result = parse(url)
        self.rowdata = []
        self.descriptions = {}
        
        for entry in result['entries']:
            enclosures = entry.get('enclosures')
            links = entry.get('links')
            url = "..."
            
            if enclosures:
                url = enclosures[0].get('url')
            else:
                url = links[0].get('href')
            
            
            title = entry.get('title',"")
            title = unihack(title)                        
            row = (title, url)
            key = hash(row)
            self.rowdata.append(row)
            self.descriptions[key] = ("<font size='2'>" + str(entry.get('description', "")) + "</font>", url)
            
        self.list.Populate(self.rowdata)


if __name__ == "__main__":
    app = wx.PySimpleApp()
    frame = wx.Frame(None, -1, '', size=(800,500))

    ppp = RSSPanel(frame, BTConfig())
    ppp.Show(True)
    frame.Show(True)

    app.MainLoop()

