#-----------------------------------------------------------------------------
# Name:        g3taskbar.py
# Purpose:     
#
# Author:      Jeremy Arendt
#
# Created:     2004/26/03
# RCS-ID:      $Id: g3taskbar.py $
# Copyright:   (c) 2002
# Licence:     See G3.LICENSE.TXT
#-----------------------------------------------------------------------------
import wx
import sys

class G3TaskBar(object):
    def __init__(self):
        if (sys.platform == 'win32'):
            wx.EVT_ICONIZE(self, self.OnMinimize)
        self.tbicon = wx.TaskBarIcon()
        
    def CreateIcon(self):
        gauge = self.btconfig.g_colors['g_overall_color']
        requested = self.btconfig.g_colors['g_requested_color']
        nothave = self.btconfig.g_colors['g_nothave_color']
        
        bmp = self.images.GetImage('g3_systray')
        dc = wx.BufferedDC(None, bmp)
        dc.BeginDrawing()
        dc.SetPen(wx.Pen(gauge, 0, wx.TRANSPARENT))
        
        try:
            dwn_fraction = self.total_drate / self.btconfig.Get('net_max_downrate')
            up_fraction = self.total_urate / self.btconfig.Get('net_max_uprate')
        except ZeroDivisionError:
            dwn_fraction = 0
            up_fraction = 0
                       
        # left side - downloads
        dc.SetBrush(wx.Brush(nothave,wx.SOLID))
        dc.DrawRectangle(0, 0, 3, 16 )
        
        dc.SetBrush(wx.Brush(gauge,wx.SOLID))
        h = int(16*min(1,dwn_fraction))
        dc.DrawRectangle(0, 16-h, 3, h)
        
        # right side - uploads
        dc.SetBrush(wx.Brush(nothave,wx.SOLID))
        dc.DrawRectangle(13, 0, 3, 16 )
        
        dc.SetBrush(wx.Brush(requested,wx.SOLID))
        h = int(16*min(1,up_fraction))
        dc.DrawRectangle(13, 16-int(16*min(1,up_fraction)), 3, h)
        
        dc.EndDrawing()

        dc = None
        icon = wx.Icon('', wx.BITMAP_TYPE_BMP)
        icon.CopyFromBitmap(bmp)
        return icon

    def OnTaskBarPopup(self, event):
        menu = wx.Menu()
        if not hasattr(self, '_tbpopupid'):
            _tbpopupid = [0] * 2
            _tbpopupid[0] = wx.NewId()
            _tbpopupid[1] = wx.NewId()
            wx.EVT_MENU(self.tbicon, _tbpopupid[0], self.OnClose)
            wx.EVT_MENU(self.tbicon, _tbpopupid[1], self.OnTaskBarActivate)
        menu.Append(_tbpopupid[0], "Quit", "", False)    
        menu.Append(_tbpopupid[1], "Restore", "", False)
        self.tbicon.PopupMenu(menu)
        
    def OnTaskBarActivate(self, event=None):
        print 'on activate'
        if 1:   #self.btconfig.Get('usesystray') < 2:
            self.tbicon.RemoveIcon()
        if self.IsIconized():
            self.Iconize(False)
        if not self.IsShown():
            self.Show(True)
        self.Raise()
        self.UpdateGUI()
    
    def OnMinimize(self, event):
        if not event.Iconized():
            self.OnTaskBarActivate()
            return
        
        print 'on minimize'
        icon = self.CreateIcon()

        if self.btconfig.Get('usesystray'):
            if not self.tbicon.IsIconInstalled():
                self.tbicon.SetIcon(icon, "G3 Torrent")
                wx.EVT_TASKBAR_LEFT_DCLICK(self.tbicon, self.OnTaskBarActivate)
                wx.EVT_TASKBAR_RIGHT_DOWN(self.tbicon, self.OnTaskBarPopup)
            self.Hide()
        else:
            self.Iconize(True)  # Iconize seems to retrigger wx.EVT_ICONIZE
    

    def UpdateIcon(self):
        icon = self.CreateIcon()
        if self.tbicon.IsIconInstalled():
            self.tbicon.SetIcon(icon, "G3 Torrent (%.0f:Dn - %.0f:Up)" % (self.total_drate/1024, self.total_urate/1024))
        else:
            self.SetIcon(icon)