#!/usr/bin/python
#-----------------------------------------------------------------------------
# Name:        g3torrent.py
# Purpose:     
#
# Author:      Jeremy Arendt
#
# Created:     2004/28/01
# RCS-ID:      $Id: g3torrent.py $
# Copyright:   (c) 2002
# Licence:     See G3.LICENCE.TXT
#-----------------------------------------------------------------------------

import os
import wx
import time
import sys
from threading import Event, Thread, Lock, enumerate
from os.path import dirname, join, split, splitext, exists, abspath
from traceback import print_exc

from btconfig import BTConfig
from images import Images
from g3widgets import *
from masterlist import MasterList
from graphpanel import GraphPanel
from detailspanel import DetailsPanel
from statuspanel import StatusPanel
from msgpanel import MessagePanel
from friendspanel import FriendsPanel
from g3taskbar import G3TaskBar
from g3version import prog_version
import friend
from g3peerid import CreatePeerId
from peerlistdns import PeerListDNS
from g3rpcserver import G3RPCServer
from BitTorrent.bencode import bencode, bdecode
from btsession import BTSession
from g3rss import RSSPanel
import encodings
import locale


#import gettext
debug_flag = False
prog_name = "G3 Torrent"
_ = lambda x: x # until I implement gettext

wxEVT_INVOKE = wx.NewEventType()
EVT_INVOKE = wx.PyEventBinder(wxEVT_INVOKE, 0)


class InvokeEvent(wx.PyEvent):
    def __init__(self, func, args, kwargs):
        wx.PyEvent.__init__(self)
        self.SetEventType(wxEVT_INVOKE)
        self.func = func
        self.args = args
        self.kwargs = kwargs

class AyUIDropTarget(wx.FileDropTarget):
    def __init__(self, window):
        wx.FileDropTarget.__init__(self)
        self.w = window
    
    def OnDropFiles(self, x, y, filenames):
        for file in filenames:
            print type(file)
            self.w.AddFromFile(file)


class AyUI(wx.Frame, G3TaskBar):
    def __init__(self, path, params):
        wx.Frame.__init__(self, None, -1, prog_name + ' ' + prog_version, size = (666,480),
            style = wx.DEFAULT_FRAME_STYLE)
        G3TaskBar.__init__(self)
        panel = wx.Panel(self, -1)
        self.panel = panel
        self.panel.SetSize(self.GetClientSize())
        self.btsessions = []
        self.btsessions_lock = Lock()
        self.uiflag = Event()
        self.doneflag = Event()
        self.last_update_time = 0
        self.btconfig = BTConfig(path)
        self.btconfig.LoadOptions()
        self.cfg_visflag = Event()
        self.path = path
        self.total_urate = 0
        self.total_drate = 0
        self.showing_error = False
        self.last_taskbar_update = 0
        
        self.last_gui_update = 0
        self.last_urate_throttle = 0
        self.last_connect_friends = 0
        self.last_broadcast = 0
        self.last_tdirscan = 0
        
##        self.lang = {}
##        gettext.install('g3torrent', './locale', unicode=True) 
##        self.lang['en'] = gettext.translation("g3torrent", './locale', languages=['en'], fallback=True) 
##        self.lang['du'] = gettext.translation("g3torrent", './locale', languages=['du'], fallback=True) 
##        self.lang['du'].install()
        
        pos = self.btconfig.Get('clientpos')
        size = self.btconfig.Get('clientsize')
        if pos[0] > 0 and pos[1] > 0:
            self.SetPosition(pos)
        if size[0] > 100 and size[1] > 100:
            self.SetSize(size)
        if self.btconfig.Get('maximized'):
            self.Maximize(True)
        
        self.images  = Images(path)
        self.pl_dnsloop = PeerListDNS(self.doneflag)
        
        self.friends = friend.FriendList(self.btconfig.Get("path"))
        self.friends.Load()
        
        self.peer_id = CreatePeerId(self.btconfig.Get("nickname"), self.btconfig.Get('client_type'))
        
        # Set frame as drop target to accept files dropped from explorer, and url text
        dt = AyUIDropTarget(self)
        self.SetDropTarget(dt)
        
        # splitter window
        splitter = SplitterWin(panel, self.btconfig)
        self.splitter = splitter
                
        # Tool Bar
        toolbar = AFToolBar(self)
        toolbar.SetToolBitmapSize((24,24))
        
        toolbar.AddTool(101, self.images.GetImage('btfile24.png'), wx.NullBitmap, False, "", _("Add From *.torrent"), _("Add new torrent from file"))
        toolbar.AddTool(102, self.images.GetImage('bturl24.png'), wx.NullBitmap, False, "", _("Add From URL"), _("Add new torrent from url"))
        toolbar.AddTool(111, self.images.GetImage('new24.png'), wx.NullBitmap, False, "", _("Create New Torrent"), _("Create New Torrent"))
        toolbar.AddSeparator()
        toolbar.AddTool(103, self.images.GetImage('remove24.png'), wx.NullBitmap, False, "", _("Remove"), _("Remove selected torrent"))
        toolbar.AddTool(104, self.images.GetImage('resume24.png'), wx.NullBitmap, False, "", _("Resume/Start"), _("Resume/Start torrent"))
        toolbar.AddTool(105, self.images.GetImage('pause24.png'), wx.NullBitmap, False, "", _("Pause/Requeue)"), _("Pause/Requeue torrent"))
        toolbar.AddTool(110, self.images.GetImage('stop24.png'), wx.NullBitmap, False, "", _("Stop"), _("Stop this torrent"))
        toolbar.AddSeparator()
        toolbar.AddTool(106, self.images.GetImage('upqueue24.png'), wx.NullBitmap, False, "", _("Move up in queue"), _("Move up in queue"))
        toolbar.AddTool(107, self.images.GetImage('downqueue24.png'), wx.NullBitmap, False, "", _("Move down in queue"), _("Move down in queue"))
        toolbar.AddSeparator()
        toolbar.AddTool(108, self.images.GetImage('config24.png'), wx.NullBitmap, False, "", _("Preferences"), _("Preferences"))
        toolbar.AddTool(109, self.images.GetImage('toggle24.png'), wx.NullBitmap, True, "", _("Toggle Splitter"), _("Toggle Splitter"))

        wx.EVT_TOOL(self, 101, self.NewTorrent_File)
        wx.EVT_TOOL(self, 102, self.NewTorrent_URL)
        wx.EVT_TOOL(self, 111, self.CreateTorrent)
        wx.EVT_TOOL(self, 103, self.KillSelected)
        wx.EVT_TOOL(self, 104, self.ResumeSelected)
        wx.EVT_TOOL(self, 105, self.PauseSelected)
        wx.EVT_TOOL(self, 110, self.StopSelected)
        wx.EVT_TOOL(self, 106, self.OnQueueUp)
        wx.EVT_TOOL(self, 107, self.OnQueueDown)
        wx.EVT_TOOL(self, 108, self.OnPreferences)
        wx.EVT_TOOL(self, 109, splitter.OnToggleSplitter)

        toolbar.Realize()
        self.SetToolBar(toolbar)
        self.toolbar = toolbar        
        
        # Menu Bar
        menuBar = wx.MenuBar()
        menu1 = wx.Menu()
        menu1.Append(201, _("&Add Torrent\t+"), _("Open/Add torrent file"))
        menu1.Append(210, _("Add Torrent (No &default save)\t*"), _("Open/Add torrent file (No default save)"))
        menu1.Append(202, _("Add Torrent From &URL\tIns"), _("Open/Add torrent file from url"))
        menu1.AppendSeparator()
        menu1.Append(211, _("&Create New Torrent\tCtrl+N"), _("Create New .torrent file"))
        menu1.AppendSeparator()
        menu1.Append(203, _("&Remove Torrent\tCtrl+Del"), _("Remove torrent file"))
        menu1.AppendSeparator()
        menu1.Append(204, _("&Q&uit\tCtrl+Q"), _("I was afraid to go on..."))
        menuBar.Append(menu1, _("&File"))

        menu1 = wx.Menu()
        menu1.Append(205, _("&Preferences\tCtrl+P"), _("Global Preferences"), False)
        menu1.AppendSeparator()
        menu1.Append(212, _("&Toggle Tab Pane\tCtrl+T"), _("Show/Hide Pane 2: The hiding of Tab Payne"), True)
        menu1.Append(209, _("Toggle &Splitter\tCtrl+S"), _("Toggle splitter"), True)
        menu1.Append(206, _("&Error Popups"), _("Toggle warning messages"), True)
        if self.btconfig.Get('popup_errors'):
            menu1.Check(206, True)
        menuBar.Append(menu1, _("&Options"))
        self.options_menu = menu1

        menu1 = wx.Menu()
        menu1.Append(207, _("A&bout\tCtrl+A"), _("About"))
        menu1.Append(208, _("Project &Homepage\tCtrl+H"), _("Project Homepage"))
        menuBar.Append(menu1, _("&Help"))
        self.SetMenuBar(menuBar)
        
        wx.EVT_MENU(self, 201, self.NewTorrent_File)
        wx.EVT_MENU(self, 210, self.NewTorrent_File_GetPath)
        wx.EVT_MENU(self, 202, self.NewTorrent_URL)
        wx.EVT_MENU(self, 211, self.CreateTorrent)
        wx.EVT_MENU(self, 203, self.KillSelected)
        wx.EVT_MENU(self, 204, self.OnClose)
        wx.EVT_MENU(self, 205, self.OnPreferences)
        wx.EVT_MENU(self, 209, splitter.OnToggleSplitter)
        wx.EVT_MENU(self, 206, self.OnErrorPopupToggle)
        wx.EVT_MENU(self, 207, self.OnAbout)
        wx.EVT_MENU(self, 208, self.OnOpenHomePage)
        wx.EVT_MENU(self, 212, self.OnToggleNotebook)
        
        # Status Bar
        self.statusbar = StatusBar_wImages(self, self.images)
        self.SetStatusBar(self.statusbar)

        border = wx.BoxSizer(wx.VERTICAL)
                    
        # Master List
        self.list = MasterList(splitter, 
                                 self.UpdateGUI,
                                 self.ResumeSelected,
                                 self.PauseSelected,
                                 self.StopSelected,
                                 self.KillSelected,
                                 self.MoveSessionUp,
                                 self.MoveSessionDown,
                                 self.CmdAnnounce,
                                 self.ShowTorrentDlg, 
                                 self.DLOrderCatcher,
                                 self.TrackerMngr,
                                 self.OnSelectInMasterList,
                                 self.ChangeSessionCfg,
                                 self.btconfig, bmps = self.images)

        # Notebook
        self.notebook = NoteBook(splitter, self.images, self.UpdateGUI)
                 
        # Panels used in the notebook
        self.status = StatusPanel(self.notebook, self.btconfig, self.images, 
                self.pl_dnsloop.GetAddress, self.FriendCatcher)
        
        self.details = DetailsPanel(self.notebook, self.btconfig, self.images)

        self.graph = GraphPanel(self.notebook, self.btconfig, self.images)
        
        self.friendsCtrl = FriendsPanel(self.notebook, self.btconfig, 
                self.friends, self.images)
        
        self.msgpanel = MessagePanel(self.notebook, self.btconfig)

        self.rsspanel = RSSPanel(self.notebook, self.btconfig, addfunc=self.AddFromURL)
        
        self.notebook.Populate(self.status, self.details, self.graph,
                 self.friendsCtrl, self.msgpanel, self.rsspanel, self.statusbar)
        
        # Set splitter
        self.splitpos = self.btconfig.Get('splitterpos')
        if self.btconfig.Get("splitter_style") == 0:
            splitter.SplitHorizontally(self.list, self.notebook, self.splitpos)
        else:
            splitter.SplitVertically(self.list, self.notebook, self.splitpos)
        if not self.btconfig['show_pane2']:
            splitter.Unsplit(self.notebook)
            
        border.Add(splitter, -1, wx.EXPAND)
        panel.SetSizer(border)
        panel.SetAutoLayout(True)
        panel.Layout()
               
        wx.EVT_SIZE(self, self.OnReSize)
        wx.EVT_CLOSE(self, self.OnClose)
        EVT_INVOKE(self, self.OnInvoke)
        wx.EVT_IDLE(self, self.OnIdle)
        
        # Only if EVT_IDLE doesnt work
        #wx.EVT_TIMER(self, -1, self.OnIdle)
        #self.timer = wx.Timer(self)
        #self.timer.Start(100)        
        
        self.SetIcon(self.images.GetImage('tbicon'))
        
        self.Show(True)
        self.OnInit()

        try:
            self.LoadWorkState()
            if len(params) > 0:
                self.AddFromFile(params[0])
        except:
            # send message here about error during startup
            pass
    
    def AntiFlicker(self, event):
        pass

    def OnUpdateUI(self, event):
        print "updateui", time.time()

    def OnToggleNotebook(self, event):
        if self.notebook.IsShown():
            self.splitter.Unsplit(self.notebook)
            self.splitpos = self.splitter.GetSashPosition()
            self.btconfig['show_pane2'] = False
        else:
            self.notebook.Show(True)
            if self.btconfig.Get("splitter_style") == 0:
                self.splitter.SplitHorizontally(self.list, self.notebook, self.splitpos)
            else:
                self.splitter.SplitVertically(self.list, self.notebook, self.splitpos)
            self.btconfig['show_pane2'] = True

    
    def OnAbout(self, event):
        pos = self.ClientToScreen( (70,30) )
        from optiondlg import About_Box
        a = About_Box(self, self.btconfig, self.images, pos )
        #self.notebook.SetSelection(5)


    def OnOpenHomePage(self, event):
        import webbrowser
        webbrowser.open_new("http://g3torrent.sourceforge.net/")
    
    
    def OnErrorPopupToggle(self, event):
        self.btconfig.Set('popup_errors', event.IsChecked())
        self.SyncSessionOptions()


    def ChangeSessionCfg(self, id, option_class):
        """ Open DLG and change options for the session that matches this id """
        if not self.cfg_visflag.isSet():
            pos = self.ClientToScreen( (50,20) )
            s = self.GetSession(id)
            if s != None:
                from optiondlg import SingleOption
                option = SingleOption(self, pos, s.GetCfg(), option_class, self.images, self.cfg_visflag)


    def OnPreferences(self, event):
        """ Open preferences dialog """
        if not self.cfg_visflag.isSet():
            from optiondlg import Options
            pos = self.ClientToScreen( (50,20) )
            self.optionswin = Options(self, pos, self.btconfig, self.images, 
                            self.cfg_visflag, syncfunc = self.SyncSessionOptions)
                            
    
    def OnReSize(self, event):
        self.panel.SetSize(self.GetClientSize())


    def OnInit(self):
        # Threads ahoy
        self.rpcserver = G3RPCServer(self.btconfig, self.AddStatMsg, self.FriendCatcher, 
            self.RemoteAddCatcher, self.btsessions, self.btsessions_lock, self.images)
        self.rpcserver.Start()
        self.pl_dnsloop.Start()       
       
        
    def OnStatMessage(self, name, msg, mtype=0):
        self.msgpanel.AddMsg(name, msg, mtype)
        self.statusbar.SetStatus(msg)
    

    def OnError(self, sess_id, msg, mtype=-1, code=None):
        if self.btconfig.Get('popup_errors') and not self.showing_error:
            self.showing_error = True
            dlg = wx.MessageDialog(self, message = msg, 
                caption = 'Download Error', style = wx.OK | wx.ICON_ERROR)
            dlg.Fit()
            dlg.Center()
            dlg.ShowModal()
            self.showing_error = False
            dlg.Destroy()

        s = self.GetSession(sess_id)
        if s != None and s.GetStaticData() != None:
            filename = split(s.GetStaticData()['responsefile'])[1]
            self.msgpanel.AddMsg(filename, msg, mtype)

            if code == 'tracker_bcool':     # Tracker flunked. But the torrent is still going...
                #Every body be cool! You, be cool!
                pass            
            elif code == 'tracker_refuse':  # Tracker refused announce attempt
                s.error_count['tracker_refuse'] += 1 
                if 'ban' in msg and s.error_count['tracker_refuse'] <= 2:
                    self.msgpanel.AddMsg(filename, "Attemping ban evasion. Attempt #%d." % \
                            s.error_count['tracker_refuse'], mtype)
                    s.SetPeerId(CreatePeerId('', 1))
                    s.Restart()
                elif 'blacklisted' in msg and s.error_count['tracker_refuse'] <= 2:
                    self.msgpanel.AddMsg(filename, "Reconfiguring settings to "
                            + "use ports 6900-6999. Attempt #%d." % \
                            s.error_count['tracker_refuse'], mtype)
                    s.ChangePortRange(6900,6999)
                elif s.error_count['tracker_refuse'] >= self.btconfig['max_trkrflunks']:
                    s.Stop()
                    self.msgpanel.AddMsg(filename, "Auto-Stopping this torrent. " \
                                    "The tracker could not be contacted", mtype)
            elif code == 'tracker_timeout':  # Tracker announce Timed out
                s.error_count['tracker_timeout'] += 1
                if s.error_count['tracker_timeout'] >= self.btconfig['max_trkrflunks']:
                    s.error_count['tracker_timeout'] = 0
                    s.Pause()
                    self.msgpanel.AddMsg(filename, "Auto-Pausing this torrent. " \
                                    "The tracker timed out after %d attempts" % \
                                        s.error_count['tracker_timeout'], mtype)
            elif code == 'tracker_404':      # Could not connect to tracker
                s.error_count['tracker_404'] += 1
                if s.error_count['tracker_404'] >= self.btconfig['max_trkrflunks']:
                    s.error_count['tracker_404'] = 0
                    s.Pause()
                    self.msgpanel.AddMsg(filename, "Auto-Pausing this torrent. " \
                                "The tracker could not be contacted after %d attempts" % \
                                    s.error_count['tracker_404'], mtype)
            
            elif code == 'hash_flunked':     # Hash flunked
                try:
                    ip = msg.split(" ")[3]
                    if not s.error_count.has_key(ip):
                        s.error_count[ip] = 1
                    else:
                        s.error_count[ip] += 1
                        
                    if s.error_count[ip] >= self.btconfig['max_hashflunks']:
                        self.FriendCatcher(ip, "Junker", friend.ADDFOETEMP, None)
                        s.ReChoke()
                        self.msgpanel.AddMsg(filename, "Temp banning %s. " \
                                    "This peer sent %d invalid pieces." % \
                                        (ip, s.error_count[ip]), mtype)
                        del s.error_count[ip]
                except:
                    print_exc()
                
        else:
            self.msgpanel.AddMsg(_("Error"), msg, -2)
        self.statusbar.SetStatus(_("An error has occured. Check message tab."))
    
    
    def OnSessionUpdate(self, sess_id, d):
        if d != None:
            s = self.GetSession(sess_id)
            if s != None: 
                s.SetStatusData(d)
        
    
    def OnSessionStart(self, sess_id, d):
        self.SaveWorkState()
        
        if d != None:
            s = self.GetSession(sess_id)
            if s != None:
                s.SetStaticData(d)
        
    
    def OnInvoke(self, event):
        if not self.uiflag.isSet() and not self.doneflag.isSet():
            apply(event.func, event.args, event.kwargs)

    
    def CmdAnnounce(self, id, url=None):
        s = self.GetSession(id)
        s.ReAnnounce(url)
    
    
    def OnQueueUp(self, event=None):
        try:
            index = self.list.GetFirstSelected()
            while index != -1:                
                id = self.list.GetItemData(index)                
                self.MoveSessionUp(id)
                index = self.list.GetNextSelected(index)
            self.UpdateGUI()
        except:
            print_exc()
    
    
    def OnQueueDown(self, event=None):
        try:
            count = 0
            index = self.list.GetFirstSelected()
            while index != -1:
                count +=1
                index = self.list.GetNextSelected(index)
            
            index = self.list.GetFirstSelected()
            while index != -1:
                id = self.list.GetItemData(index)
                self.MoveSessionDown(id, count)
                index = self.list.GetNextSelected(index)
            self.UpdateGUI()
        except:
            print_exc()
    
    
    def MoveSessionUp(self, id):
        index = self.GetSessionIndex(id)
        if index == None:
            return

        if index > 0:
            session = self.btsessions.pop(index)
            self.btsessions.insert(index-1, session)
                    
    
    def MoveSessionDown(self, id, nselected):
        index = self.GetSessionIndex(id)
        if index == None:
            return

        session = self.btsessions.pop(index)
        self.btsessions.insert(index+nselected, session)
    
    
    def MoveSessionToBack(self, id):
        index = self.GetSessionIndex(id)
        if index == None:
            return
        
        session = self.btsessions.pop(index)            
        self.btsessions.append(session)
    
    
    def MoveSessionToFront(self, id):
        index = self.GetSessionIndex(id)
        if index == None:
            return
        
        session = self.btsessions.pop(index)            
        self.btsessions.insert(0, session)
                    
    
    def GetSessionIndex(self, id):
        i = 0
        for s in self.btsessions:
            if s.GetId() == id:
                return i
            i += 1
        return None

    
    def InvokeLater(self, func, args = [], kwargs = {}):
        if not self.uiflag.isSet() and not self.doneflag.isSet():
            wx.PostEvent(self, InvokeEvent(func, args, kwargs))
    
    
    def ScrapeCatcher(self, id, data):
        def foo(self=self, id=id, data=data):
            s = self.GetSession(id)
            if s != None:
                s.scrape_data = data
                self.UpdateGUI()

        self.InvokeLater(foo, [])
        
            
    def FriendCatcher(self, ip, peerid, msgtype, data):
        def foo(self=self):
            self.friends.GotUpdate(ip, peerid, msgtype, data)
            self.friendsCtrl.Update()

        self.InvokeLater(foo, [])
    
    
    def RemoteAddCatcher(self, params, mtype=0):
        if mtype == 0:
            self.OnTaskBarActivate()
            self.InvokeLater(self.AddFromFile, [params])
        elif mtype == 1:
            self.OnTaskBarActivate()
            self.InvokeLater(self.AddFromURL, [params])
    
    def AddStatMsg(self, name, msg, mtype=0):
        self.InvokeLater(self.OnStatMessage, [name, msg, mtype])

    
    def ErrorCatcher(self, id, msg, code=None):
        self.InvokeLater(self.OnError, [id, msg, -1, code])
            
    
    def UpdateCatcher(self, id, d):
        self.InvokeLater(self.OnSessionUpdate, [id, d])

    
    def DNSCatcher(self, id, d):
        self.InvokeLater(self.OnSessionUpdate, [id, d])
        
    
    def StartCatcher(self, id, d):
        self.InvokeLater(self.OnSessionStart, [id, d])
       
    
    def DLOrderCatcher(self, id, manual=False):
        self.InvokeLater(self.DLOrderMngrDlg, [id, manual])
                
    
    def OnIdle(self, event):
        # UpdateGUI every 'gui_update_rate'
        if self.last_gui_update + max(0.25, self.btconfig['gui_update_rate']) < time.time():
            self.UpdateGUI()
            
            if not self.IsShown() and self.last_taskbar_update + 4 < time.time():
                self.UpdateIcon()
                self.last_taskbar_update = time.time()
            self.last_gui_update = time.time()
        
        # Throttle urate every 1 sec
        if self.last_urate_throttle + 1 < time.time():
            self.URateThrottle()
            self.last_urate_throttle = time.time()
            
        # ConnectFriends every 60 sec
        if self.last_connect_friends + 60 < time.time():
            self.ConnectFriends()
            self.last_connect_friends = time.time()
            
        # Broadcast infohashes to friends every 60 sec
        if self.last_broadcast + 60 < time.time():
            self.Broadcast()
            self.last_broadcast = time.time()

        # Search torrent folder for new torrents every N mins
        if self.btconfig['tdir_scan'] != 0 and self.btconfig['use_torrent_dir'] and self.btconfig['use_download_dir']:
            if self.last_tdirscan + max(60, self.btconfig['tdir_scan']) < time.time():
                self.TorrentDirScan()
                self.last_tdirscan = time.time()
    
    
    def TorrentDirScan(self):
        torrentdir = self.btconfig['torrent_dir']
        if not exists(torrentdir):
            return
        
        added_torrents = []
        for s in self.btsessions:
            if s.GetResponseFilename():
                added_torrents.append( split(s.GetResponseFilename())[1] )
        
        torrentfiles = os.listdir(torrentdir)
        for torrent in torrentfiles:
            responsefile =  join(torrentdir, torrent)
            
            if splitext(responsefile)[1] != ".torrent":
                continue
            try:
                added_torrents.index(torrent)
            except ValueError:
                print 'Loading from file %s' % torrent
                added_torrents.append(torrent)
                self.AddFromFile(responsefile)
                        
    
    def ResetStatus(self, id):
        self.status.peerlist.Reset()
        self.status.pppgauge.SetValueUnknown()


    def TrackerMngr(self, id):
        s = self.GetSession(id)
        if s != None and s.IsRunning():
            from trackermngr import TrackerDlg
            tmngr = TrackerDlg(self, id, self.btconfig, self.images, 
                        s.download.SetAnnounceURLs, self.CmdAnnounce)
            tmngr.Populate( s.download.GetAnnounceURLs() )
            tmngr.Show()
    
                    
    def DLOrderMngrDlg(self, id, manual=False):
        s = self.GetSession(id)
        if s != None and (manual or self.btconfig['use_multimngr']):
            from dlordermngr import DLOrderMngr
            manager = DLOrderMngr(self, self.btconfig, self.images, 
                    s.GetFileRanges(), s.IsDLOrderRandom(), s.SetPieceRanges)
     
        
    def ShowTorrentDlg(self, id):
        s = self.GetSession(id)
        if not s.spew and s != None and s.GetStatusData() != None:
            from btdownloadgui import DownloadInfoFrame
            s.spew = True
            f = DownloadInfoFrame(self, s, self.images, 
                        self.pl_dnsloop.GetAddress, self.FriendCatcher)
            
            
    def UpdateGUI(self):
        if not self.btsessions_lock.acquire(False):
            return
        
        selected_id = self.list.GetSelectedData()
        selected = None
        total_urate = 0.0
        total_drate = 0.0
        nActiveSessions = 0
        nWaitingSessions = 0
        nStoppedSessions = 0
        nCheckingSessions = 0
        
        for s in self.btsessions:
            if not s.IsDone() and s.IsRunning():
                if s.IsChecking():
                    nCheckingSessions += 1
                nActiveSessions += 1                              
            elif not s.IsDone() and s.IsPaused() and s.IsStopped():
                nStoppedSessions += 1
            elif not s.IsDone() and s.IsPaused() and not s.IsChecking():
                nWaitingSessions += 1
        
        i = 0
        while i < len(self.btsessions):
            s = self.btsessions[i]
            s.SetQRank(i)
            
            if selected_id == s.GetId():
                selected = s
            
            # not FAILED or entered DONE state
            if not s.IsDone():
                # if in RUNNING state
                if s.IsRunning() and s.GetStatusData() != None:
                    urate = s.GetStat('urate')
                    drate = s.GetStat('drate')
                        
                    if urate:
                        total_urate += s.GetStat('urate')
                    
                    if drate:
                        total_drate += s.GetStat('drate')
                    
                    s.AppendRateRecord(drate, urate)
                    
                    try:
                        self.list.AddSession(s)
                    except:
                        print_exc()
                        print 'ERROR: error appending master list'
                        
                    #0: stop only queue>0, 1: stop on cust option, 2: keep sharing, 3: stop
                    if s.IsComplete():
                        cfg = s.GetCfg()
                        if cfg.Get('on_complete') == 0:
                            if nWaitingSessions > 0:
                                nWaitingSessions -= 1
                                s.Stop()
                        elif cfg.Get('on_complete') == 1:
                            if s.GetFilesize() and s.GetStat('utotal'):
                                dtotal = s.GetFilesize()
                                utotal = s.GetStat('utotal')
                                ratio =  float(utotal) / dtotal
                            else:
                                ratio = 0

                            if cfg.Get('end_on_newrate'):
                                cfg.Set('use_global_urate', False)
                                cfg.Set('maxupspeed', cfg['end_newrate']/1024)
                                s.GetDownload().SetMaxUpSpeed(cfg['end_newrate'])
                            if cfg.Get('end_on_percent') and ratio >= cfg.Get('end_percent'):
                                s.Stop()
                            if cfg.Get('end_on_ratio') and ratio >= cfg.Get('end_ratio'):
                                s.Stop()
                            if cfg.Get('end_on_timelimit') and (s.GetFinTime() + cfg.Get('end_timelimit')) <= time.time():
                                s.Stop()
                                
                        elif cfg.Get('on_complete') == 2:
                            pass # do nothing
                        elif cfg.Get('on_complete') == 3:
                            s.Stop()

                # if in PAUSED state
                elif s.IsPaused():
                    try:
                        if s.GetStatusData() != None:
                            self.list.AddSession(s)
                        else:
                            self.list.AddPlaceholder(s)
                    except:
                        print_exc()
                        print 'ERROR: error appending master list'
                                
                    # try to start or unpause next session in queue, unless was paused manually
                    if nCheckingSessions < 1 and nActiveSessions < self.btconfig.Get('max_sessions') and not s.IsStopped():
                        print 'trying to resume ' + str(s.GetId())
                        s.Resume()
                        nActiveSessions += 1
                        nCheckingSessions += 1
                    
                # EXCEPTION/LIMBO STATE
                else:                
                    # stuff in the limbo state still needs to be displayed
                    try:
                        if s.GetStatusData() != None:
                            self.list.AddSession(s)
                        else:
                            self.list.AddPlaceholder(s)
                    except:
                        print_exc()
                        print 'ERROR: error appending master list'
                i += 1
            # DONE or FAILED state
            else:
                try:
                    self.list.RemoveSession(s)
                    self.btsessions.remove(s)
                    s.Kill()
                    self.SaveWorkState()
                except:
                    print str(s.GetId()) + ' ERROR: error removing from master list'

        #end while
        
        # enter data into graph every cycle
        self.graph.Append_DRate(total_drate)
        self.graph.Append_URate(total_urate)
        self.total_urate = total_urate
        self.total_drate = total_drate

        self.UpdateStatusPanels(selected)
        
        self.list.BreakItDown()
        
        #update statusbar
        self.statusbar.SetUpRate(total_urate/1024)
        self.statusbar.SetDownRate(total_drate/1024)
        
        self.btsessions_lock.release()
    
    
    def URateThrottle(self):
        if not self.btconfig['use_global_urate']: 
            return
        
        cur_total_urate = 0
        n_sessions = 0
        largest_urate = 0
        for s in self.btsessions:
            if s.IsRunning():
                rate = s.GetStat('urate')
                if rate != None and rate > 1:
                    n_sessions += 1
                    cur_total_urate += rate
                    if rate > largest_urate:
                        largest_urate = rate

        if n_sessions <= 0:
            return

        max_urate = self.btconfig['total_max_uprate']
        if max_urate == 0:
            rate = 0
        else:
            slack = max(0, max_urate - cur_total_urate)
            rate =  min(max_urate, int((max_urate / n_sessions) + slack))

        for s in self.btsessions:
            s_data = s.GetStatusData()
            if not s.IsRunning() or s.IsChecking() or s_data == None:
                continue
            
            cfg = s.GetCfg()
            if not cfg['use_global_urate']:
                continue

            target_peer_rate = 1024 * cfg['avg_peer_urate']            
            cur_rate = s_data['urate']
            if cur_rate == None:
                cur_rate = 0
            
            if (cur_rate > rate or cfg['maxupspeed'] != int(rate/1024)) and \
                    (rate + cfg['maxupspeed']) != 0:
                s.GetDownload().SetMaxUpSpeed(rate)
                cfg['maxupspeed'] = int(rate/1024)
            
            if rate > 0:
                maxuploads = max(1, int(round(0.5 + float(rate) / max(1024, target_peer_rate))))
            else:
                maxuploads = max(4, int(round(0.5 + float(cur_rate) / max(1024, target_peer_rate))))

            if largest_urate > target_peer_rate:
                maxuploads += 1
            elif largest_urate + 2048 < target_peer_rate and maxuploads > 1:
                maxuploads -= 1
            
            s.GetDownload().SetMaxUploads( maxuploads )
            cfg['maxuploads'] = maxuploads


    def UpdateStatusPanels(self, selected):
        try:
            # Display status on list selected item
            if selected != None and self.IsShown():
                d = selected.GetStatusData()
                s = selected.GetStaticData()

                if selected.IsStopped():
                    self.TogglePauseButton(True, True)
                elif selected.IsPaused():
                    self.TogglePauseButton(False)
                else:
                    self.TogglePauseButton(True)

                # Status Panel
                if self.notebook.GetSelection() == 0:
                    if selected.IsComplete():
                        self.status.pppgauge.SetValueFinished()
                    elif selected.IsPaused():
                        self.status.pppgauge.SetValuePaused()
                        self.status.peerlist.Reset()
                    elif selected.HasFailed():
                        self.status.pppgauge.SetValueFailed()
                    elif d != None:
                        pppg_time = time.time()
                        self.status.pppgauge.SetValue(d['fractionDone'], 
                                d['havelist'], d['availlist'])
                        
                    if selected.IsRunning() and d != None and s != None:
                        pop_time = time.time()
                        self.status.peerlist.Populate(d['spew'], s['info_hash'], selected.ReChoke, selected.GetCfg())
                    else:
                        self.status.peerlist.Reset()
                        
                # Details Panel
                elif self.notebook.GetSelection() == 1:
                    self.details.Update(selected)
                # Friends Panel
                elif self.notebook.GetSelection() == 3:
                    self.friendsCtrl.Update()
            
            # Nothing selected
            elif self.IsShown():
                if self.notebook.GetSelection() == 0:
                    self.status.pppgauge.SetValueUnknown()
                    self.status.peerlist.Populate()
                
            # Graph Panel section
            if self.notebook.GetSelection() == 2:
                if selected != None and selected.IsRunning() and \
                    selected.GetStatusData() != None:
                    self.graph.AttachSession(selected)
                    d = selected.GetStatusData()
                    self.graph.Append_Sel_DRate(d['drate'])
                    self.graph.Append_Sel_URate(d['urate'])
                else:
                    self.graph.PushAsunder()
                if self.IsShown():
                    self.graph.UpdateGraph()
                else:
                    self.graph.PushAsunder()
            else:
                self.graph.PushAsunder()

            # Main list
            if self.IsShown():
                self.list.Sort()
                
                for s in self.btsessions:
                    if s is not selected and not s.spew:
                        s.ToggleSpew(False)
                    else:
                        s.ToggleSpew(True)
            else:
                for s in self.btsessions:
                    if not s.spew:
                        s.ToggleSpew(False)
                    else:
                        s.ToggleSpew(True)

        except:
            print_exc()

    
    def ConnectFriends(self):
        for s in self.btsessions:
            xdata = s.GetStaticData()
            if s.IsRunning() and xdata:
                infohash = xdata['info_hash']
                interested = self.friends.GetNeedsConnection(infohash)
                                
                for ip, port, peerid in interested:
                    s.StartConnection(ip, port, peerid)
    
    
    def Broadcast(self):
        live_files = []

        for s in self.btsessions:
            if s.IsRunning() and s.GetStaticData() != None:
                live_files.append([ s.GetStaticData()['info_hash'],
                                    s.GetStaticData()['listen_port'],
                                    s.GetStaticData()['peer_id'] ])
    
        self.friends.Broadcast(live_files)
        self.last_friend_announce = time.time()
    
    
    def OnSelectInMasterList(self, id):
        selected = self.GetSession(id)
        #self.status.peerlist.Reset()
        try:
            self.UpdateGUI()
        except:
            print_exc()
    
    
    def TogglePauseButton(self, status, both=False):
        if both:
            self.toolbar.EnableTool(104, status)
            self.toolbar.EnableTool(105, status)
        else:
            self.toolbar.EnableTool(104, not status)
            self.toolbar.EnableTool(105, status)


    def PauseSelected(self, event=None):
        try:
            self.SavePeers()
            index = self.list.GetFirstSelected()
            while index != -1:
                id = self.list.GetItemData(index)
                self.PauseTorrent(id)
                index = self.list.GetNextSelected(index)
            self.UpdateGUI()
        except:
            print_exc()
    
    def ResumeSelected(self, event=None):
        try:
            index = self.list.GetFirstSelected()
            while index != -1:
                id = self.list.GetItemData(index)
                self.ResumeTorrent(id)
                index = self.list.GetNextSelected(index)
            self.UpdateGUI()
        except:
            print_exc()
                
    def StopSelected(self, event=None):
        try:
            index = self.list.GetFirstSelected()
            while index != -1:
                id = self.list.GetItemData(index)
                self.StopTorrent(id)
                index = self.list.GetNextSelected(index)
            self.UpdateGUI()
            self.SavePeers()
        except:
            print_exc()
                    

    def StopTorrent(self, id):
        if id != None:
            s = self.GetSession(id)
            if s != None:
                s.Stop()
                    
                    
    def PauseTorrent(self, id):
        if id != None:
            s = self.GetSession(id)
            if s != None:
                if not s.IsPaused():
                    s.Pause()
                    self.TogglePauseButton(True)
                    self.MoveSessionToBack(id)
                elif s.IsStopped():
                    s.Pause()
                    self.TogglePauseButton(True)                 
    
    def ResumeTorrent(self, id):
        if id != None:
            s = self.GetSession(id)
            if s != None:
                s.Resume()
                self.TogglePauseButton(False)
                self.MoveSessionToFront(id) 
                    
    def NewSessionId(self):
        if not hasattr(self, '_last_id'):
            self._last_id = 100
        else:
            self._last_id += 1
        return self._last_id


    def AddTorrent(self, params = None, confirm_dir=False, btconfig=None):
        self.AddStatMsg(_("General"), _("Adding Torrent"))
        try:
            id = self.NewSessionId()
            
            if btconfig == None:
                btconfig = self.btconfig.Clone()
                
            print 'session ' + str(id) + ' Adding torrent'
            session = BTSession(self, id, self.peer_id, self.InvokeLater, 
                    self.UpdateCatcher, self.ErrorCatcher, self.StartCatcher, 
                    self.friends.GetInterested, btconfig, self.images, 
                    params, confirm_dir, self.AddStatMsg, self.DLOrderCatcher, self.path)
            if not session.IsDone():
                self.btsessions.append(session)
                self.UpdateGUI()
            else:
                self.AddStatMsg(_("Error"), _("Could not start download"), -2)
                print 'ERROR: Could not start download'
        except:
            print_exc()

        
    def AddFromURL(self, url):
        self.AddStatMsg(_("General"), _("Trying to load from URL %s") % url)
        print "trying to load from URL %s" % url
        
        if self.btconfig.Get('use_torrent_dir'):
            path = self.btconfig.Get('torrent_dir')
        else:
            #need some temp place to put this
            path = self.path
        
        from urlfetch import T_FetchFile
        geturl = T_FetchFile(self.InvokeLater, self.AddFromFile, url, path)
        geturl.start()
        return True


    def AddFromFile(self, responsefile):
        from BitTorrent.download import Download
        # do checks then add
        self.AddStatMsg(_("General"), _("Trying to load from file %s") % responsefile)
        print "trying to load from FILE %s" % responsefile.encode('latin-1', 'ignore')
        d = Download()
        response = d.ParseResponseFile(responsefile)
        if response == None:
            self.AddStatMsg(_("Error"), _("Response file %s contains invalid data") % responsefile, -2)
            print 'ERROR: responsefile contains invalid data'
            return False
        
        info = response['info']
        filename = info['name']

        # attept to see if this torrent is already added
        for s in self.btsessions:
            if s.GetResponseFilename() == None:
                continue
            if split(s.GetResponseFilename())[1] == split(responsefile)[1]:
                self.AddStatMsg(_("Error"), _("A response file named %s is already loaded - ignoring") % split(responsefile)[1], -2)
                return True
            if s.GetFilename() == filename:
                # To do. make this optional via the preferences dialog
                self.AddStatMsg(_("Error"), _("A file named %s is already loaded - ignoring") % filename, -2)
                #self.AddTorrent( [responsefile], True )
                return True


        # check in the completed folder and retarget DL
        dldir = self.btconfig['download_dir']
        comp_dldir = self.btconfig['completed_dl_dir']
        if self.btconfig['use_download_dir'] and exists(dldir) and exists(comp_dldir) and abspath(dldir) != abspath(comp_dldir):
            
            files = os.listdir(comp_dldir)
            for f in files:
                if f == filename:
                    # add confirmation here
                    print 'Found matching filename in completed folder'
                    startup_p = ["--responsefile", responsefile, "--saveas", join(comp_dldir, filename)]
                    btconfig = self.btconfig.Clone()
                    self.AddTorrent(startup_p, False, btconfig)
                    return True

        self.AddTorrent( [responsefile] )
        return True


    def SaveWorkState(self):
        import ConfigParser
        print 'saving work state'
        
        size = self.GetSize()
        pos = self.GetPosition()
        
        if sys.platform == 'win32':
            maximized = self.IsMaximized()
            self.btconfig['maximized'] = maximized
            if not maximized:
                self.btconfig['clientpos'] = pos
                self.btconfig['clientsize'] = size

        splitpos = self.splitter.GetSashPosition()        
        if splitpos >= self.splitter.GetMinimumPaneSize():
            self.btconfig['splitterpos'] = splitpos
        
        self.btconfig.SaveOptions()
        
        cp = ConfigParser.ConfigParser()
        statefile = join(self.path, 'torrents.ini')
        cp.add_section('TORRENTS')

        ordered_sessions = [(s.GetQRank(), s) for s in self.btsessions]
        ordered_sessions.sort()

        for rank, s in ordered_sessions:
            try:
                params = s.GetSaveParams()
                if len(params) > 0:
                    cp.set('TORRENTS', str(rank), str(params))
            except:
                print_exc()
                print 'ERROR saving %s' % s.GetFilename()
        
        file = open(statefile, 'w')
        cp.write(file)
        file.close()
    
    
    def SavePeers(self):
        all_peers = []
        # save peers for 
        for s in self.btsessions:
            if s.IsRunning() and s.GetStat('spew'):
                rec = []
                rec.append(s.GetFilename())
                rec.append(s.GetSavePeers())
                all_peers.append(rec)
        
        all_peers = bencode(all_peers)
        peerfile = join(self.path, 'peers.ben')
        file = open(peerfile, 'wb')
        file.write(all_peers)
        file.close()

    
    def LoadWorkState(self):
        import ConfigParser
        self.AddStatMsg(_("General"), _("Loading work state"))
        print 'loading work state'
        statefile = join(self.path, 'torrents.ini')
        if not exists(statefile):
            return
        
        torrentdir = self.btconfig.Get('torrent_dir')
        completeddir = self.btconfig.Get('completed_tor_dir')
        load_completed = self.btconfig.Get('load_completed')
        
        # load what was saved in the ini file, 
        # except what is in completed dir if load_completed is False
        cp = ConfigParser.ConfigParser()
        
        try:
            file = open(statefile, 'r')
            cp.readfp(file)
            file.close()
            items = cp.items('TORRENTS')
            cfg = {}
            added_torrents = []
            sorted_items = []

            #for some lame reason, items is in an arbitrary order
            for key, params in items:
                sorted_items.append( (int(key), params) )
                
            sorted_items.sort()
            
            for key, params in sorted_items:
                p = eval(params)
                try:
                    responsefile = p[p.index("--responsefile") + 1]
                    saveas = p[p.index("--saveas") + 1]
                    cfg = p[p.index("--cfg") + 1]
                    up_total = p[p.index("--up_total") + 1]
                    down_total = p[p.index("--down_total") + 1]
                    check_hashes = p[p.index("--check_hashes") + 1]
                except ValueError:
                    continue
                if split(responsefile)[0] == completeddir and not load_completed:
                    continue
                else:
                    print 'from ini '  + split(responsefile)[1]
                    startup_p = []
                    startup_p.append( "--responsefile" )
                    startup_p.append( responsefile )
                    startup_p.append( "--saveas" )
                    startup_p.append( saveas )
                    startup_p.append( "--up_total" )
                    startup_p.append( up_total )
                    startup_p.append( "--down_total" )
                    startup_p.append( down_total )
                    if not self.btconfig['always_check_hashes']:
                        startup_p.append( "--check_hashes" )
                        startup_p.append( check_hashes )
                    
                    added_torrents.append(split(responsefile)[1])
                    cfg = eval(cfg)
                    btconfig = self.btconfig.Clone()
                    btconfig.Inject(cfg)
                    self.AddTorrent(startup_p, False, btconfig)
        except:
            print_exc()
        
        # load torrents in torrent folder, unless already loaded.
        if not self.btconfig.Get('use_torrent_dir') or not exists(torrentdir):
            return
        
        torrentfiles = os.listdir(torrentdir)
        for torrent in torrentfiles:
            print "trying to load %s" % torrent
            responsefile =  join(torrentdir, torrent)
            
            if splitext(responsefile)[1] != ".torrent":
                continue
            try:
                added_torrents.index(torrent)
            except ValueError:
                print 'Loading from file %s' % torrent
                added_torrents.append(torrent)
                self.AddFromFile(responsefile)


    def CreateTorrent(self, event):
        from btcompletedirgui import CreateTorrentDlg
        pos = self.ClientToScreen( (50,20) )
        frame = CreateTorrentDlg(self, self.btconfig, pos, self.AddFromFile)
        frame.Show(True)


    def NewTorrent_File(self, event):
        self.AddTorrent()
    
    
    def NewTorrent_File_GetPath(self, event):
        self.AddTorrent(confirm_dir=True)
    
    
    def NewTorrent_URL(self, event):
        data = None
        cb = wx.Clipboard()
        if cb.Open():
            data = wx.TextDataObject()
            cb.GetData(data)
            cb.Close()
            
        dlg = wx.TextEntryDialog(self, 'Enter torrent URL','Add From URL', 'G3 Torrent')
        if data and "torrent" in data.GetText() or "http" in data.GetText():
            dlg.SetValue(data.GetText())
        else:
            dlg.SetValue("")
        if dlg.ShowModal() == wx.ID_OK:
            self.AddFromURL(dlg.GetValue())
        dlg.Destroy()
        
        
    def GetSession(self, id):
        for s in self.btsessions:
            if s.GetId() == id:
                return s
        return None
    
    
    def SyncSessionOptions(self, dosync=False):
        if dosync:
            self.AddStatMsg(_("General"), _("Syncing options"))
            for s in self.btsessions:
                try:
                    cfg = s.GetCfg()
                    cfg.MirrorOptions(self.btconfig)
                except:
                    print_exc()
        else:
            self.options_menu.Check(206, self.btconfig['popup_errors'])

    
    def KillTorrent(self, id, death_mode=-1):
        s = self.GetSession(id)
        del_torrent = False
        del_data = False
        remove_dlg = None
        
        if death_mode > -1:
            apply2all = True
        else:
            apply2all = False
        
        if death_mode == -1:
            from optiondlg import RemoveTorrentDlg
            remove_dlg = RemoveTorrentDlg(self, btconfig = BTConfig(), bmps=self.images)
            death_mode = remove_dlg.ShowModal()
            remove_dlg.Destroy()
            if death_mode == wx.ID_CANCEL:
                return None
            if remove_dlg.Apply2All():
                apply2all = True
        
        if death_mode == 1:
            del_torrent = True
        elif death_mode == 2:
            del_data = True
        elif death_mode == 3:
            del_torrent = True
            del_data = True

        responsefile = s.GetResponseFilename()
        if s.GetDownload() and s.GetDownload().GetFiles():
            datafiles = s.GetDownload().GetFiles()
        else:
            del_data = False

        # delete torrent file
        if del_torrent:
            if responsefile != None:
                try:
                    self.AddStatMsg(_("General"), _("Deleting torrent file %s") % responsefile, 0)
                    os.remove(responsefile)
                except OSError:
                    self.AddStatMsg(_("Error"), _("Could not delete file %s") % responsefile, -2)
        else:
            if split(responsefile)[0] == abspath(self.btconfig.Get('torrent_dir')):
                try:
                    os.rename(responsefile, responsefile+"_ABORTED")
                except OSError:
                    pass
        
        # delete data
        if del_data:
            if s != None:
                s.Kill()

            nfiles = 0
            for filename, len in datafiles:
                nfiles += 1
                try:
                    self.AddStatMsg(_("General"), _("Deleting file %s") % filename, 0)
                    os.remove(filename)
                except OSError:
                    self.AddStatMsg(_("Error"), _("Could not delete file %s") % filename, -2)
                   
            if nfiles > 1:
                try:
                    os.rmdir( dirname( datafiles[0][0] ) )
                except:
                    self.AddStatMsg(_("Error"), _("Could not remove dir %s") % dirname( datafiles[0][0] ), -2)
                    
            s = None

        if s != None:
            s.Kill()
        
        self.UpdateGUI()       
        if apply2all:
            return death_mode
        return -1

            
    def KillSelected(self, event):
        self.AddStatMsg(_("General"), _("Killing selected torrent(s)"))
        print 'Killing selected torrent(s)'
        selected = []
        index = self.list.GetFirstSelected()

        while index != -1:
            id = self.list.GetItemData(index)
            selected.append(id)
            index = self.list.GetNextSelected(index)
        try:
            death_mode = -1
            for id in selected:
                death_mode = self.KillTorrent(id, death_mode)
                if death_mode == None:
                    break
        except:
            print_exc()
            
        self.UpdateGUI()

    
    def OnClose(self, event):
        # get confirmation before closing
        if self.btconfig.Get('confirmexit'):
            dlg = wx.MessageDialog(self, 'Do you really want to exit?',
                          'Confirmation', wx.YES_NO | wx.ICON_INFORMATION)
            dlgresult = dlg.ShowModal()
            dlg.Destroy()
            
            if dlgresult != wx.ID_YES:
                return
        try:
            if self.cfg_visflag.isSet():
                return
            print 'On Close'
            #self.timer.Stop()
            self.pl_dnsloop.Stop()
            self.rpcserver.Stop()
            self.Show(False)
            self.SaveWorkState()
            self.SavePeers()
            self.doneflag.set()
            
            print 'Killing sessions ' + str(len(self.btsessions))
            for s in self.btsessions:
                s.Kill()
        except:
            print 'ERROR: Did not shut down correctly'
            print_exc()
            
        print 'Destroying window'
        self.Destroy()
        print enumerate()

        
class MyApp(wx.App):
    def __init__(self, flag):
        wx.App.__init__(self, flag)
        wx.Log_SetActiveTarget(wx.LogStderr())
        self.done = True
        
    def MainLoop(self):
        if "wxMac" in wx.PlatformInfo:
            # TODO:  Does wxMac implement wxEventLoop yet???
            wx.App.MainLoop()
        else:
            evtloop = wx.EventLoop()
            old = wx.EventLoop.GetActive()
            wx.EventLoop.SetActive(evtloop)

            while not self.doneflag.isSet():
                while evtloop.Pending():
                    evtloop.Dispatch()

                time.sleep(0.10)
                self.ProcessIdle()

            wx.EventLoop.SetActive(old)
                   
    def OnInit(self):
        path = dirname(sys.argv[0])
        frame = AyUI(path, sys.argv[1:])
        self.SetTopWindow(frame)
        self.doneflag = frame.doneflag
        return True

def popup_abort():
    app = wx.PySimpleApp()
    frame = wx.Frame(None, -1, '')
    dlg = wx.MessageDialog(frame, 'Another instance of %s is already running' % prog_name,
                          'Abort', wx.OK | wx.ICON_INFORMATION)
    dlg.ShowModal()
    dlg.Destroy()
    frame.Destroy()

def run(params):
    try: 
        app = MyApp(False)
        app.MainLoop()
    except:
        print_exc()
        
   
if __name__ == '__main__':
    print "G3 Torrent requires wxwindows 2.5.2.7" 
    print "Installed wx version: %s" % wx.__version__
    if debug_flag:
        print 'debugging'
        run(sys.argv[1:])
    else:
        checker = wx.SingleInstanceChecker( prog_name + '-' + wx.GetUserId() )
        if checker.IsAnotherRunning() == False:
            run(sys.argv[1:])
        else:
            print 'Another instance is already running'
            if len(sys.argv[1:]) > 0:
                from g3rpcclient import G3RPCCLient
                rpc = G3RPCCLient()
                rpc.AddTorrent(sys.argv[1:][0])
            else:
                popup_abort()
