#-----------------------------------------------------------------------------
# Name:        g3widgets.py
# Purpose:     
#
# Author:      Jeremy Arendt
#
# Created:     2004/23/02
# RCS-ID:      $Id: g3widgets.py $
# Copyright:   (c) 2002
# Licence:     See G3.LICENCE
#-----------------------------------------------------------------------------
import wx
import sys
import time
import os.path
from g3unihack import *
from btconfig import BTConfig
from p2pgauge import P2P_Gauge, Avail_Gauge
_ = lambda x: x

class NoteBook(wx.Notebook):
    def __init__(self, parent, images, updateguifunc):
        wx.Notebook.__init__(self, parent, -1, size = (-1, -1))
        
        self.images = images
        self.UpdateGUI = updateguifunc
        
        wx.EVT_NOTEBOOK_PAGE_CHANGING(parent, self.GetId(), self.TabChanging) 
        wx.EVT_NOTEBOOK_PAGE_CHANGED(parent, self.GetId(), self.TabChanged) 
        
    def Populate(self, statuspanel, detailspanel, graphpanel, friendspanel, 
            msgpanel, rsspanel, statusbar):
        self.graphpanel = graphpanel
        self.statusbar = statusbar
        
        bmps = self.images
        
        #mask_color = wx.Color(0,0,0)
        #b_mask_color = wx.Color(0,0,0)
##        self.images = [1] * 6
##        self.i_list = wx.ImageList(16, 16)
##        self.images[0] = self.i_list.Add(bmps.GetImage('msg16.png'))
##        self.images[1] = self.i_list.Add(bmps.GetImage('msg16.png'))
##        self.images[2] = self.i_list.Add(bmps.GetImage('msg16.png'))
##        self.images[3] = self.i_list.Add(bmps.GetImage('smile.bmp'))
##        self.images[4] = self.i_list.Add(bmps.GetImage('msg16.png'))
##       
##        self.SetImageList(self.i_list)
        
        self.AddPage(statuspanel, _("Status"), imageId=0)
        self.AddPage(detailspanel, _("Details"), imageId=1)
        self.AddPage(graphpanel, _("Graph"), imageId=2)
        self.AddPage(friendspanel, _("Friends"), imageId=3)
        self.AddPage(msgpanel, _("Messages"), imageId=4)
        self.AddPage(rsspanel, _("RSS Reader"), imageId=4)
        
    
    def TabChanged(self, event):
        selection =  event.GetSelection()
        self.GetPage(selection).Show(True)
        if selection == 2:
            self.graphpanel.UpdateSize()
            self.graphpanel.UpdateGraph()
        if selection == 4:
            self.statusbar.SetStatus("")
        self.UpdateGUI()


    def TabChanging(self, event):
        if sys.platform == "win32":
            oldpage = event.GetOldSelection()
            if oldpage >= 0:
                self.GetPage(oldpage).Show(False)
            
                
class AFToolBar(wx.ToolBar):
    def __init__(self, parent, id=-1, size=wx.DefaultSize, style=(wx.TB_HORIZONTAL|wx.NO_BORDER|wx.TB_FLAT|wx.TB_DOCKABLE)):
        wx.ToolBar.__init__(self, parent, id, size = size, style = style)
        wx.StaticLine(self, -1, pos = (96, 3), size = (2, 24), style = wx.LI_VERTICAL)
        wx.StaticLine(self, -1, pos = (228, 3), size = (2, 24), style = wx.LI_VERTICAL)
        wx.StaticLine(self, -1, pos = (298, 3), size = (2, 24), style = wx.LI_VERTICAL)
        wx.EVT_SIZE(self, self.OnReSize)
    
    def OnReSize(self, event):
        self.Refresh(False)
        self.Update()
        
class SplitterWin(wx.SplitterWindow):
    def __init__(self, parent, btconfig):
        wx.SplitterWindow.__init__(self, parent, -1, style=wx.SP_NOBORDER )
        wx.EVT_SIZE(self, self.OnReSize)
        wx.EVT_SPLITTER_DCLICK(self, -1, self.OnToggleSplitter) 
        
        self.parent = parent
        self.btconfig = btconfig
        #if minpane is not large enough wxListCtrlAutoWidthMixin freaks out
        self.SetMinimumPaneSize(100)
    
    def OnReSize(self, event):
        event.Skip()
        if self.GetSashPosition() > 0:
            self.SetSashPosition(self.GetSashPosition())
            
    def OnToggleSplitter(self, event):
        if self.GetSplitMode() == wx.SPLIT_HORIZONTAL:
            self.SetSplitMode(wx.SPLIT_VERTICAL)
            self.btconfig.Set('splitter_style', 1)
        else:
            self.SetSplitMode(wx.SPLIT_HORIZONTAL)
            self.btconfig.Set('splitter_style', 0)

        self.parent.SetSize((self.parent.GetSize()[0]+1, self.parent.GetSize()[1]))
        self.parent.SetSize((self.parent.GetSize()[0]-1, self.parent.GetSize()[1]))        
                        
class StatusBar_wImages(wx.StatusBar):
    def __init__(self, parent, images):
        wx.StatusBar.__init__(self, parent, -1, style = wx.ST_SIZEGRIP)
        self.SetFieldsCount(3)
        self.SetStatusWidths([-1, 150, 160])
        self.upbmp = wx.StaticBitmap(self, -1, images.GetImage('up16.png'))
        self.downbmp = wx.StaticBitmap(self, -1, images.GetImage('down16.png'))
        wx.EVT_SIZE(self, self.OnSize) 
        self.OnSize()
        
    def SetStatus(self, msg):
        msg = unihack(msg)
        self.SetStatusText(msg)
    
    def SetUpRate(self, rate):
        self.SetStatusText("       Total Up: %.1f KB/s" % rate , 2)
            
    def SetDownRate(self, rate):
        self.SetStatusText("       Total Down: %.1f KB/s" % rate , 1)
    
    def OnSize(self, event=None):
        rect = self.GetFieldRect(1)
        self.downbmp.SetPosition(wx.Point(rect.x+1, rect.y+1))
        rect = self.GetFieldRect(2)
        self.upbmp.SetPosition(wx.Point(rect.x+1, rect.y+1))
        
        
class Balloon(wx.Frame):
    def __init__(self, parent, pos=wx.DefaultPosition, size=(180,85)):
        wx.Frame.__init__(self, parent, -1, "Alert", pos, size,
                style=wx.FRAME_SHAPED|wx.SIMPLE_BORDER|wx.FRAME_NO_TASKBAR)
        panel = wx.Panel(self, -1)
        self.panel = panel
        panel.SetSize(size)
        self.SetSize(size)
        
        panel.SetBackgroundColour(wx.Color(255,255,225))
        info_bmp = wx.ArtProvider_GetBitmap(wx.ART_TIP, wx.ART_MESSAGE_BOX, (16,16))
        sb = wx.StaticBitmap(panel, -1, info_bmp, (8,4))
        
        st1 = wx.StaticText(panel, -1, "Info", (30,4))
        st1.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))
        
        st2 = wx.StaticText(panel, -1, "000000000000000000000000000000", (30,20))
        st2.SetSize((size[0]-40, size[1]-10))
       
        wx.EVT_LEFT_DOWN(st1, self.OnClose)
        wx.EVT_LEFT_DOWN(st2, self.OnClose)
        wx.EVT_LEFT_DOWN(sb, self.OnClose)
        wx.EVT_LEFT_DOWN(panel, self.OnClose)
       
        if wx.Platform != "__WXMAC__":

            pass



        if wx.Platform == "__WXGTK__":

            self.Bind(wx.EVT_WINDOW_CREATE, self.SetFrameShape)

        else:

            self.SetFrameShape()

        
        self.SetFrameShape()
        self.Show()
    
    
    def SetFrameShape(self, event=None):
        size = self.GetSize()
        boxheight = size.y - 30
        boxwidth = size.x
        
        bmp = wx.EmptyBitmap(size.x,size.y)
        dc = wx.BufferedDC(None, bmp)
        dc.BeginDrawing()
        dc.SetBackground(wx.Brush( wx.Color(0,0,0), wx.SOLID))
        dc.Clear()
        dc.SetPen(wx.Pen(wx.Color(0,0,0), 1, wx.TRANSPARENT))
        dc.SetBrush(wx.Brush(wx.Color(255,0,255), wx.SOLID))
        
        dc.DrawRoundedRectangle( 0,0, boxwidth,boxheight, 12 )
        dc.DrawPolygon( ((boxwidth-40,boxheight), (boxwidth-20,boxheight+20), (boxwidth-20,boxheight)) ) 
        dc.EndDrawing()

        r = wx.RegionFromBitmapColour(bmp, wx.Color(0,0,0))
        self.hasShape = self.SetShape(r)
            
    def OnClose(self, event):
        self.Destroy()
            

class Balloon_wTimer(Balloon):
    def __init__(self, parent, pos=wx.DefaultPosition, size=(180,85), timeout=10000):
        Balloon.__init__(self, parent, pos, size)
        wx.EVT_TIMER(self, -1, self.OnTimer)
        
        self.timer = wx.Timer(self)
        self.timer.Start(timeout)
        
    def OnTimer(self, event):
        self.timer.Stop()
        self.Close()


# Wide with upward poiting angle
class MLDetailsBalloon(wx.Frame):
    def __init__(self, parent, btconfig, pos=(100,100), size=(250,140),
            status="None", filename="ima.pimp", filesize=0, complete=0.0, 
                havelist=None, availlist=None):
        pos = (pos[0]-15, pos[1])
        wx.Frame.__init__(self, parent, -1, "Alert", pos, size,
                style=wx.FRAME_SHAPED|wx.NO_BORDER|wx.FRAME_NO_TASKBAR)
        panel = wx.Panel(self, -1)
        self.panel = panel
        self.bg_color = wx.Color(255,255,225)
        self.btconfig = btconfig
        
        panel.SetBackgroundColour(self.bg_color)

        if wx.Platform == "__WXMSW__":
            icon = self.GetIconFromFilename(filename)
            if icon:
                info_bmp = wx.EmptyBitmap(16,16)
                info_bmp.CopyFromIcon(icon)
            else:
                info_bmp = wx.ArtProvider_GetBitmap(wx.ART_TIP, wx.ART_MESSAGE_BOX, (16,16))
        else:
            info_bmp = wx.ArtProvider_GetBitmap(wx.ART_TIP, wx.ART_MESSAGE_BOX, (16,16))
            
        info_image = wx.StaticBitmap(panel, -1, info_bmp)
        
        filename = unihack(filename)            
        filelabel = wx.StaticText(panel, -1, filename)
        filelabel.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.BOLD))
        
        size = (max(180, filelabel.GetSize()[0]) +60, size[1])
        panel.SetSize(size)
        self.SetSize(size)

        sizelabel = wx.StaticText(panel, -1, "%s: %d%s (%0.1f/%0.1f MB)"%(status, 100*complete, 
                '%', float(filesize*complete)/(1<<20), float(filesize)/(1<<20)))
        sizelabel.SetSize((size[0]-40, -1))
        
        hline = wx.StaticLine(panel, -1)

        detaillabel = wx.StaticText(panel, -1, "Progress and Availability")
        detaillabel.SetSize((size[0]-40, -1))
        
        p2p_gauge = P2P_Gauge(panel, self.btconfig, size=(1,20), showtext=False)
        avail = Avail_Gauge(panel, self.btconfig, size=(1,20))
        
        p2p_gauge.SetValue(complete, havelist)
        avail.SetValue(complete, availlist)
        
        bsizer = wx.BoxSizer(wx.VERTICAL)
        double = wx.FlexGridSizer(cols = 2, vgap=2, hgap=8)
        shorty = wx.FlexGridSizer(cols = 1, vgap=1)
        tall = wx.FlexGridSizer(cols = 1, vgap=1)
        tall.AddGrowableCol(0)
        
        shorty.Add(filelabel, 1, wx.EXPAND)
        shorty.Add(sizelabel, 1, wx.EXPAND)
        double.AddGrowableCol(1)
        double.Add(info_image, 1, wx.EXPAND)
        double.Add(shorty, 1, wx.EXPAND)
        
        tall.Add(double, 1, wx.EXPAND)

        tall.Add((-1,2))
        tall.Add(hline, 1, wx.EXPAND)
        tall.Add((-1,2))
        tall.Add(detaillabel, 1, wx.EXPAND)
        tall.Add(p2p_gauge, 1, wx.EXPAND)
        tall.Add(avail, 1, wx.EXPAND)
        
        bsizer.Add((-1,8))
        bsizer.Add(tall, 1, wx.EXPAND|wx.ALL, 8)
        
        panel.SetSizer(bsizer)
        panel.Layout()

        wx.EVT_LEFT_DOWN(p2p_gauge, self.OnClose)
        wx.EVT_LEFT_DOWN(avail, self.OnClose)
        wx.EVT_LEFT_DOWN(filelabel, self.OnClose)
        wx.EVT_LEFT_DOWN(sizelabel, self.OnClose)
        wx.EVT_LEFT_DOWN(detaillabel, self.OnClose)
        wx.EVT_LEFT_DOWN(info_image, self.OnClose)
        wx.EVT_LEFT_DOWN(panel, self.OnClose)
        wx.EVT_ERASE_BACKGROUND(panel, self.OnErase)
        wx.EVT_ACTIVATE(self, self.SetFrameShape)
        wx.EVT_CLOSE(self, self.OnClose)
        
        self.Show(True)
        
    def OnClose(self, event):
        self.Show(False)
        self.Destroy()        
   
    def GetIconFromFilename(self, filename):
        ext = os.path.splitext(filename)[1]
        info = wx.TheMimeTypesManager.GetFileTypeFromExtension(ext)
        if info:
            icon = info.GetIcon()
            if icon and icon.Ok():
                return icon
        return None
    
    def OnErase(self, event):
        size = self.GetSize()
        boxheight = size.y - 30
        boxwidth = size.x
        
        dc = event.GetDC()
        bg_brush = wx.Brush(self.bg_color)
        dc.SetBackground(bg_brush)
        dc.Clear()
        dc.BeginDrawing()
        dc.SetPen(wx.Pen(wx.Color(0,0,0), 1, wx.SOLID))
        dc.SetBrush(wx.Brush(self.bg_color, wx.SOLID))
        dc.DrawRoundedRectangle( 0,10, boxwidth,boxheight, 8 )

        dc.DrawPolygon( ((8,10), (15,1), (22,10)))
        
        dc.SetPen(wx.Pen(self.bg_color, 2, wx.SOLID))
        dc.DrawRectangle( 10,10, 12,2)
        
        dc.EndDrawing()


    def SetFrameShape(self, event=None):
        size = self.GetSize()
        boxheight = size.y - 30
        boxwidth = size.x
        
        bmp = wx.EmptyBitmap(size.x,size.y)
        dc = wx.BufferedDC(None, bmp)
        dc.BeginDrawing()
        dc.SetBackground(wx.Brush( wx.Color(0,0,0), wx.SOLID))
        dc.Clear()
        dc.SetPen(wx.Pen(wx.Color(0,0,0), 1, wx.TRANSPARENT))
        dc.SetBrush(wx.Brush(wx.Color(255,0,255), wx.SOLID))
        
        dc.DrawRoundedRectangle( 0,10, boxwidth,boxheight, 8 )
        dc.DrawPolygon( ((8,10), (15,0), (22,10)) ) 
        dc.EndDrawing()

        r = wx.RegionFromBitmapColour(bmp, wx.Color(0,0,0))
        self.hasShape = self.SetShape(r)

    
if __name__ == "__main__":
    app = wx.PySimpleApp()
    wx.Log_SetActiveTarget(wx.LogStderr())
    f = MLDetailsBalloon(None, BTConfig())
    
    app.SetTopWindow(f)
    app.MainLoop()
    