#-----------------------------------------------------------------------------
# Name:        scrape.py
# Purpose:     
#
# Author:      Jeremy Arendt
#
# Created:     2004/03/03
# RCS-ID:      $Id: scrape.py $
# Copyright:   (c) 2002
# Licence:     MIT
#-----------------------------------------------------------------------------
import wx
from threading import Thread
from urllib2 import URLError, urlopen 
from os.path import split, join
from urlparse import urlparse
from BitTorrent.bencode import bdecode
from g3listctrl import G3ListCtrl
from wx.lib.mixins.listctrl import ColumnSorterMixin, ListCtrlAutoWidthMixin
from traceback import print_exc
import socket

class ScrapeListCtrl(G3ListCtrl, ListCtrlAutoWidthMixin):
    def __init__(self, parent, btconfig, images):
        G3ListCtrl.__init__(self, parent, btconfig, "ScrapeList")
        ListCtrlAutoWidthMixin.__init__(self)
        
        cols = [ [True, "Filename", wx.LIST_FORMAT_LEFT, 160],
                 [True, "Seeds", wx.LIST_FORMAT_RIGHT, 60],
                 [True, "Peers", wx.LIST_FORMAT_RIGHT, 60],
                 [True, "Downloads", wx.LIST_FORMAT_LEFT, wx.LIST_AUTOSIZE_USEHEADER],
                ]
        self.InsertColumns(cols)
    
    def Populate(self, data):
        pass
    
class ScrapeDlg(wx.Frame):
    def __init__(self, parent, btconfig, images, url, id=-1, invokefunc=None):
        wx.Frame.__init__(self, parent, -1, 'Scrape', size=(400,300))
        self.btconfig = btconfig
        self.images = images
        self.InvokeLater = invokefunc
        self.url = url
        
        okbut_id = wx.NewId()
        
        panel = wx.Panel(self, -1)
        self.list = ScrapeListCtrl(panel, btconfig, images)
        self.gauge = wx.Gauge(panel, -1, 100, size=(-1, 10))
        okbut = wx.Button(panel, okbut_id, "Ok")
        
        boxer = wx.BoxSizer(wx.VERTICAL)
        tall = wx.FlexGridSizer(cols = 1)
        buttons = wx.FlexGridSizer(cols = 1)
        
        buttons.Add(okbut, 1, wx.ALIGN_RIGHT)
        tall.Add(self.list, 1, wx.EXPAND)
        tall.Add(self.gauge, 1, wx.EXPAND)
        tall.Add(buttons, 1, wx.EXPAND)
        tall.AddGrowableRow(0)
        tall.AddGrowableCol(0)
        
        boxer.Add(tall, 1, wx.EXPAND | wx.ALL, 2) 
        
        panel.SetSizer(boxer)
        panel.Layout()
                
        wx.EVT_CLOSE(self, self.Close)
        self.Show(True)
        
        self.Scrape(self.url)
    
    def Scrape(self, url):
        t = T_Scrape(self.PostScrape, "", url)
        t.start()

    def PostScrape(self, params, data):
        files = data[1]
        if files == None:
            return
        
        i=0
        for hash, data in files.items():
            if data.get('name') == None: 
                name = "No name field"
            else:
                name = "%s" % data.get('name')
            downloaded = "%d" % data.get('downloaded')
            complete = "%d" % data.get('complete')
            incomplete = "%d" % data.get('incomplete')
            
            self.list.InsertRow(i, [name,complete,incomplete,downloaded] ) 
            i += 1
        
    def Close(self, event=None):
        self.Destroy()



class T_Scrape(Thread):

    def __init__(self, invokelater, callback, url, infohash=None):

        Thread.__init__(self)
        self.infohash = infohash

        self.success = False

        self.Callback = callback
        self.InvokeLater = invokelater
        self.url = url
        self.setDaemon(False)
        
    def run(self):
        try:
            h = urlopen(self.url)
        except (IOError, URLError, socket.error):
            print "ERROR: urlopen(%s) failed" % self.url
            return False

        data = None
        try:
            data = bdecode( h.read() )
            data = data['files']
            if self.infohash:
                data = data[self.infohash]
        except:
            #print_exc()
            self.success = False
            return False

        self.success = True
        try:
            self.InvokeLater(self.Callback, (self.success, data))
        except wx.PyDeadObjectError:
            pass
        return True


def dummy_callback(params1, params2):
    print params1, params2
    
    data = params2[2]

if __name__ == "__main__":
##    t = T_Scrape(dummy_callback, "", "http://tracker4.slash0.org:6969/scrape")
##    t.start()
    
    app = wx.PySimpleApp()
    frame = ScrapeDlg(None, None, None, "http://tracker4.slash0.org:6969/scrape", None)
    app.SetTopWindow(frame)
    frame.Show()
    app.MainLoop()
        
